#
# Copyright (c) 2010 Canonical
#
# Written by Gustavo Niemeyer <gustavo@niemeyer.net>
#
# This file is part of the Xpresser GUI automation library.
#
# Xpresser is free software; you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License version 3,
# as published by the Free Software Foundation.
#
# Xpresser is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import time

from gi.repository import Gtk, Gdk, GdkPixbuf

from xpresser import xutils
from xpresser.image import Image
from xpresser.opencvfinder import OpenCVFinder

from xpresser.tests.images import get_image_path
from xpresser.lib.testing import TestCase


class XUtilsTestBase(TestCase):

    def setUp(self):
        xutils.hover(0, 0)

    def flush_gtk(self):
        # Why do we need to do so much to get the button in place before
        # actually trying to click on it? :-(  If we just run until there
        # are no more events, and without sleep, the button will simply
        # return (0, 0) as its position.
        while Gtk.events_pending():
            Gtk.main_iteration()
        time.sleep(0.1) # Why oh why? :-(
        while Gtk.events_pending():
            Gtk.main_iteration()
        time.sleep(0.1)
        while Gtk.events_pending():
            Gtk.main_iteration()

    def create_window(self, child):
        window = Gtk.Window(Gtk.WindowType.TOPLEVEL)
        window.connect("delete_event", lambda widget, event: False)
        window.add(child)
        child.show()
        window.show()
        return window

    def create_button_window(self, image_path=None):
        button = Gtk.Button()
        if image_path is None:
            image_path = get_image_path("red-square.png")
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(image_path)
        image = Gtk.Image.new_from_pixbuf(pixbuf)
        button.set_image(image)
        return self.create_window(button)

    def create_image_window(self, image_path):
        pixbuf = GdkPixbuf.Pixbuf.new_from_file(image_path)
        image = Gtk.Image.new_from_pixbuf(pixbuf)
        return self.create_window(image)


class XUtilsTest(XUtilsTestBase):

    def test_type(self):
        entry = Gtk.Entry()
        window = self.create_window(entry)
        try:
            window.present()
            entry.grab_focus()
            self.flush_gtk()
            xutils.type("Hello there!")
            self.flush_gtk()
            self.assertEquals(entry.get_text(), "Hello there!")
        finally:
            window.destroy()

    def test_take_screenshot(self):
        """Verify that take_screenshot works, with a real screenshot.

        For that, we will put a known image in the screen, and will then
        try to find it in the screenshot.
        """
        red_square = Image("red-square",
                           filename=get_image_path("red-square.png"))
        window = self.create_image_window(red_square.filename)

        self.flush_gtk()

        width = Gdk.get_default_root_window().get_width()
        height = Gdk.get_default_root_window().get_height()

        window_x, window_y = window.get_child().get_window().get_position()

        time.sleep(2)
        big_screenshot = xutils.take_screenshot()

        window.destroy()
        self.flush_gtk()

        # Check the basic attributes set
        self.assertEquals(big_screenshot.name, "screenshot")
        self.assertEquals(big_screenshot.width, width)
        self.assertEquals(big_screenshot.height, height)

        # Now verify the actual images taken.
        finder = OpenCVFinder()

        big_match = finder.find(big_screenshot, red_square)

        self.assertEquals(big_match.image, red_square)
        self.assertTrue(big_match.similarity > 0.95, big_match.similarity)

        # The match we found in the big screenshot should be in the same
        # position as the window we created.  Note that this may fail if
        # you have the image opened elsewhere. ;-)
        self.assertEquals(big_match.x, window_x)
        self.assertEquals(big_match.y, window_y)


class XUtilsButtonTest(XUtilsTestBase):

    def setUp(self):
        super(XUtilsButtonTest, self).setUp()
        self.window = self.create_button_window()
        self.button = self.window.get_child()
        self.button_clicked = False
        self.button_rclicked = False
        self.button_hovered = False
        self.button_dclicked = False

        def clicked(widget, event):
            if event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
                self.button_dclicked = True
            elif event.button == 1 and event.type == Gdk.EventType.BUTTON_PRESS:
                self.button_clicked = True
            elif event.button == 3:
                self.button_rclicked = True

        def entered(widget):
            self.button_hovered = True

        self.button.connect("button_press_event", clicked)
        self.button.connect("enter", entered)

        self.flush_gtk()

    def tearDown(self):
        self.window.destroy()

    def get_button_center(self):
        button_x, button_y = self.button.get_window().get_position()
        button_width = self.button.get_window().get_width()
        button_height = self.button.get_window().get_height()
        return (button_x + button_width//2, button_y + button_height//2)
        self.window.destroy()

    def test_click(self):
        xutils.click(*self.get_button_center())
        self.flush_gtk()
        self.assertTrue(self.button_clicked)
        self.window.destroy()

    def test_right_click(self):
        xutils.right_click(*self.get_button_center())
        self.flush_gtk()
        self.assertTrue(self.button_rclicked)
        self.window.destroy()

    def test_double_click(self):
        xutils.double_click(*self.get_button_center())
        self.flush_gtk()
        self.assertTrue(self.button_dclicked)
        self.window.destroy()

    def test_hover(self):
        xutils.hover(*self.get_button_center())
        self.flush_gtk()
        self.assertTrue(self.button_hovered)
        self.assertFalse(self.button_clicked)
